from dataclasses import dataclass, field
from datetime import datetime
from typing import Dict, List


@dataclass
class Measure:
    accumulate: int
    instant: float


@dataclass
class PeriodValue:
    start: datetime
    end: datetime
    value: float


@dataclass
class ConsumptionForPeriod(PeriodValue):
    desglosed: dict[str, float] = field(default_factory=dict)


@dataclass
class HistoricalConsumption:
    periods: list[ConsumptionForPeriod] = field(default_factory=list)
    total: float = 0
    desglosed: dict[str, float] = field(default_factory=dict)


@dataclass
class HistoricalGeneration:
    periods: list[PeriodValue] = field(default_factory=list)


@dataclass
class DemandAtInstant:
    dt: datetime
    value: float


@dataclass
class HistoricalPowerDemand:
    demands: list[DemandAtInstant] = field(default_factory=list)



# Copyright (C) 2021-2022 Luis López <luis@cuarentaydos.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
# USA.


import functools
import json
import logging
from datetime import datetime, timedelta
from typing import Any

import aiohttp

from . import parsers
from .types import (
    HistoricalConsumption,
    HistoricalGeneration,
    HistoricalPowerDemand,
    Measure,
)

_BASE_URL = "https://www.i-de.es/consumidores/rest"

#
# URLs not confirmed since begining of the times
#
_CONTRACTS_ENDPOINT = f"{_BASE_URL}/cto/listaCtos/"
_CONTRACT_DETAILS_ENDPOINT = f"{_BASE_URL}/detalleCto/detalle/"
_CONTRACT_SELECTION_ENDPOINT = f"{_BASE_URL}/cto/seleccion/"
_GENERATION_PERIOD_ENDPOINT = (
    f"{_BASE_URL}/consumoNew/obtenerDatosGeneracionPeriodo/"
    "fechaInicio/{start:%d-%m-%Y}00:00:00/"
    "fechaFinal/{end:%d-%m-%Y}00:00:00/"
)
_ICP_STATUS_ENDPOINT = f"{_BASE_URL}/rearmeICP/consultarEstado"
_LOGIN_ENDPOINT = f"{_BASE_URL}/loginNew/login"
_MEASURE_ENDPOINT = f"{_BASE_URL}/escenarioNew/obtenerMedicionOnline/24"

#
# URLs reviewed on 2023-06-22
#
_CONSUMPTION_PERIOD_ENDPOINT = (
    f"{_BASE_URL}/consumoNew/obtenerDatosConsumoDH/"
    "{start:%d-%m-%Y}/"
    "{end:%d-%m-%Y}/"
    "horas/USU/"
)

_POWER_DEMAND_LIMITS_ENDPOINT = f"{_BASE_URL}/consumoNew/obtenerLimitesFechasPotencia/"
_POWER_DEMAND_PERIOD_ENDPOINT = (
    f"{_BASE_URL}/consumoNew/obtenerPotenciasMaximasRangoV2/"
    # fecMin and fecMax are provided by _POWER_DEMAND_LIMITS_ENDPOINT
    "{fecMin}/{fecMax}"
)


async def get_session() -> aiohttp.ClientSession:
    return aiohttp.ClientSession()


def auth_required(fn):
    @functools.wraps(fn)
    async def _wrap(client, *args, **kwargs):
        if client._auto_renew_user_session is True and client.is_logged is False:
            await client.login()

        return await fn(client, *args, **kwargs)

    return _wrap


class Client:
    _HEADERS = {
        "Content-Type": "application/json; charset=utf-8",
        "esVersionNueva": "1",
        "idioma": "es",
        "movilAPP": "si",
        "tipoAPP": "ios",
        "User-Agent": (
            "Mozilla/5.0 (iPhone; CPU iPhone OS 11_4_1 like Mac OS X) "
            "AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15G77"
        ),
    }

    def __init__(
        self,
        session: aiohttp.ClientSession,
        username: str,
        password: str,
        contract: str | None = None,
        logger: logging.Logger | None = None,
        user_session_timeout: timedelta | int = 300,
        auto_renew_user_session: bool = True,
    ):
        if not isinstance(user_session_timeout, timedelta):
            user_session_timeout = timedelta(seconds=user_session_timeout)

        self._sess = session
        self._username = username
        self._password = password
        self._contract = contract
        self._logger = logger or logging.getLogger("ideenergy")
        self._user_session_timeout = user_session_timeout
        self._auto_renew_user_session = auto_renew_user_session

        self._login_ts: datetime | None = None

    #
    # Some properties
    #

    @property
    def username(self) -> str:
        return self._username

    @property
    def password(self) -> str:
        return self._password

    @property
    def is_logged(self) -> bool:
        if not self._login_ts:
            return False

        delta = datetime.now() - self._login_ts
        return delta < self.user_session_timeout

    @property
    def user_session_timeout(self) -> timedelta:
        return self._user_session_timeout

    @property
    def auto_renew_user_session(self) -> bool:
        return self._auto_renew_user_session

    #
    # Requests
    #

    async def _request(self, method: str, url: str, **kwargs) -> aiohttp.ClientResponse:
        headers = kwargs.get("headers", {})
        headers.update(self._HEADERS)
        kwargs["headers"] = headers

        resp = await self._sess.request(method, url, **kwargs)
        if resp.status != 200:
            raise RequestFailedError(resp)

        return resp

    async def request_bytes(self, method: str, url: str, **kwargs) -> bytes:
        resp = await self._request(method, url, **kwargs)
        buff = await resp.content.read()
        return buff

    async def request_json(
        self, method: str, url: str, encoding: str = "utf-8", **kwargs
    ) -> dict[Any, Any]:
        buff = await self.request_bytes(method, url, **kwargs)
        data = json.loads(buff.decode(encoding))
        return data

    #
    # Methods
    #

    async def login(self) -> None:
        """
        {
            'redirect': 'informacion-del-contrato',
            'zona': 'B',
            'success': 'true',
            'idioma': 'ES',
            'uCcr': ''
        }
        """
        payload = [
            self.username,
            self.password,
            "",
            "Android 6.0",
            "Móvil",
            "Chrome 119.0.0.0",
            "0",
            "",
            "s",
            "",
        ]

        data = await self.request_json("POST", _LOGIN_ENDPOINT, json=payload)
        if not isinstance(data, dict):
            raise InvalidData(data)

        if data.get("success", "false") != "true":
            raise CommandError(data)

        self._login_ts = datetime.now()
        self._logger.info(f"successful authentication as '{self.username}'")

        if self._contract:
            await self.select_contract(self._contract)
            self._logger.info(f"contract '{self._contract}' selected ")

    @auth_required
    async def is_icp_ready(self) -> bool:
        """
        {
            'icp': 'trueConectado'
        }
        """
        data = await self.request_json("POST", _ICP_STATUS_ENDPOINT)
        ret = data.get("icp", "") == "trueConectado"

        return ret

    @auth_required
    async def get_contract_details(self) -> dict[str, Any]:
        """
        {
            "ape1Titular": "xxxxxx                                       ",
            "ape2Titular": "xxxxxx                                       ",
            "codCliente": "12345678",
            "codContrato": 123456789.0,
            "codPoblacion": "000000",
            "codProvincia": "00",
            "codPostal": "00000",
            "codSociedad": 3,
            "codTarifaIblda": "92T0",
            "codTension": "09",
            "cups": "ES0000000000000000XY",
            "direccion": "C/ XXXXXXXXXXXXXXXXXX, 12 , 34 00000-XXXXXXXXXXXXXXXXXXXX"
                " - XXXXXXXXX           ",
            "dni": "12345678Y",
            "emailFactElec": "xxxxxxxxx@xxxxx.com",
            "esAutoconsumidor": False,
            "esAutogenerador": False,
            "esPeajeDirecto": False,
            "fecAltaContrato": "2000-01-01",
            "fecBajaContrato": 1600000000000,
            "fecUltActua": "22.10.2021",
            "indEstadioPS": 4,
            "indFraElectrn": "N",
            "nomGestorOficina": "XXXXXXXXXXXX                                      ",
            "nomPoblacion": "XXXXXXXXXXXXXXXXXXXX                         ",
            "nomProvincia": "XXXXXXXXX                                    ",
            "nomTitular": "XXXXX                                        ",
            "numCtaBancaria": "0",
            "numTelefono": "         ",
            "numTelefonoAdicional": "123456789",
            "potDia": 0.0,
            "potMaxima": 5750,
            "presion": "1.00",
            "puntoSuministro": "1234567.0",
            "tipAparato": "CN",
            "tipCualificacion": "PP",
            "tipEstadoContrato": "AL",
            "tipo": "A",
            "tipPuntoMedida": None,
            "tipSectorEne": "01",
            "tipSisLectura": "TG",
            "tipSuministro": None,
            "tipUsoEnerg": "",
            "listContador": [
                {
                    "fecInstalEquipo": "28-01-2011",
                    "propiedadEquipo": "i-DE",
                    "tipAparato": "CONTADOR TELEGESTION",
                    "tipMarca": "ZIV",
                    "numSerieEquipo": 12345678.0,
                }
            ],
            "esTelegestionado": True,
            "esTelegestionadoFacturado": True,
            "esTelemedido": False,
            "cau": None,
        }
        """
        data = await self.request_json("GET", _CONTRACT_DETAILS_ENDPOINT)
        if not data.get("codContrato", False):
            raise InvalidData(data)

        return data

    @auth_required
    async def get_contracts(self) -> list[dict[str, Any]]:
        """
        {
            'success': true,
            'contratos': [
                {
                    'direccion': 'xxxxxxxxxxxxxxxxxxxxxxx',
                    'cups': 'ES0000000000000000AB',
                    'tipo': 'A',
                    'tipUsoEnergiaCorto': '-',
                    'tipUsoEnergiaLargo': '-',
                    'estContrato': 'Alta',
                    'codContrato': '123456789',
                    'esTelegestionado': True,
                    'presion': '1.00',
                    'fecUltActua': '01.01.1970',
                    'esTelemedido': False,
                    'tipSisLectura': 'TG',
                    'estadoAlta': True
                }
            ]
        }
        """
        data = await self.request_json("GET", _CONTRACTS_ENDPOINT)
        if not data.get("success", False):
            raise CommandError(data)

        try:
            return data["contratos"]
        except KeyError:
            raise InvalidData(data)

    @auth_required
    async def select_contract(self, id: str) -> None:
        resp = await self.request_json("GET", _CONTRACT_SELECTION_ENDPOINT + id)
        if not resp.get("success", False):
            raise InvalidContractError(id)

        self._contract = id
        self._logger.info(f"{self}: '{id}' contract selected")

    @auth_required
    async def get_measure(self) -> Measure:
        """
        {
            "valMagnitud": "158.64",
            "valInterruptor": "1",
            "valEstado": "09",
            "valLecturaContador": "43167",
            "codSolicitudTGT": "012345678901",
        }
        """

        self._logger.debug("Requesting data to the ICP, may take up to a minute.")
        data = await self.request_json("GET", _MEASURE_ENDPOINT)

        self._logger.debug(f"Got reply, raw data: {data!r}")

        try:
            measure = Measure(
                accumulate=int(data["valLecturaContador"]),
                instant=float(data["valMagnitud"]),
            )

        except (KeyError, ValueError) as e:
            raise InvalidData(data) from e

        self._logger.info(f"{self}: ICP measure reading successful")
        return measure

    @auth_required
    async def get_historical_consumption(
        self, start: datetime, end: datetime
    ) -> HistoricalConsumption:
        start = min([start, end])
        end = max([start, end])
        url = _CONSUMPTION_PERIOD_ENDPOINT.format(start=start, end=end)

        data = await self.request_json("GET", url, encoding="iso-8859-1")

        ret = parsers.parse_historical_consumption(data)
        ret.periods = [x for x in ret.periods if x.start >= start and x.end < end]
        return ret

    async def get_historical_generation(
        self, start: datetime, end: datetime
    ) -> HistoricalGeneration:
        start = min([start, end])
        end = max([start, end])
        url = _GENERATION_PERIOD_ENDPOINT.format(start=start, end=end)

        data = await self.request_json("GET", url, encoding="iso-8859-1")
        ret = parsers.parse_historical_generation(data)

        return ret

    # @auth_required
    # async def _get_historical_generic_data(
    #     self, url_template: str, start: datetime, end: datetime
    # ) -> Dict[Any, Any]:
    #     start = min([start, end])
    #     end = max([start, end])
    #     url = url_template.format(start=start, end=end)

    #     data = await self.request_json("GET", url, encoding="iso-8859-1")

    #     base_date = datetime(start.year, start.month, start.day)
    #     ret = parsers.parser_generic_historical_data(data, base_date)

    #     return ret

    @auth_required
    async def get_historical_power_demand(self) -> HistoricalPowerDemand:
        @auth_required
        async def _get_available_interval(client):
            url = _POWER_DEMAND_LIMITS_ENDPOINT

            data = await client.request_json("GET", url)
            assert data.get("resultado") == "correcto"

            return data

        limits = await _get_available_interval(self)
        url = _POWER_DEMAND_PERIOD_ENDPOINT.format(**limits)

        data = await self.request_json("GET", url)

        ret = parsers.parse_historical_power_demand_data(data)
        return ret

    def __repr__(self):
        return f"<ideenergy.Client username={self.username}, contract={self._contract}>"


class ClientError(Exception):
    pass


class RequestFailedError(ClientError):
    def __init__(self, response):
        self.response = response

    def __str__(self):
        return f"Invalid response: {self.response.status} - {self.response.reason}"


class CommandError(ClientError):
    def __init__(self, data):
        self.data = data

    def __str__(self):
        return f"Command not succesful: {self.data!r}"


class InvalidData(ClientError):
    def __init__(self, data):
        self.data = data

    def __str__(self):
        return f"Invalid data from server: {self.data!r}"


class InvalidContractError(ClientError):
    def __init__(self, data):
        self.data = data

    def __str__(self):
        return f"Invalid contract code: {self.data!r}"



#!/usr/bin/env python3

# Copyright (C) 2021-2022 Luis López <luis@cuarentaydos.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
# USA.


import os
import unittest
from datetime import datetime, timedelta
from unittest.mock import AsyncMock, patch

from ideenergy import Client, get_session
from ideenergy.client import _LOGIN_ENDPOINT

FIXTURES_DIR = os.path.dirname(__file__) + "/fixtures"


def read_fixture(fixture_name: str):
    with open(f"{FIXTURES_DIR}/{fixture_name}.bin", "rb") as fh:
        return fh.read()


class TestClient(unittest.IsolatedAsyncioTestCase):
    async def asyncSetUp(self):
        self.sess = await get_session()
        self.client = Client(None, "x", "y")
        self.end = datetime.now().replace(hour=0, minute=0, second=0)
        self.start = self.end - timedelta(days=7)

    async def asyncTearDown(self):
        await self.sess.close()

    async def test_login_ok(self):
        with patch(
            "ideenergy.Client.request_bytes",
            new_class=AsyncMock,
            return_value=read_fixture("login-ok"),
        ) as fn:
            await self.client.login()

        fn.assert_awaited_once()

        args, kwargs = fn.call_args
        self.assertEqual(args, ("POST", _LOGIN_ENDPOINT))

    @patch("ideenergy.Client.is_logged", return_value=True)
    async def test_historical_consumption(self, _):
        with patch(
            "ideenergy.Client.request_bytes",
            new_class=AsyncMock,
            return_value=read_fixture("historical-consumption"),
        ):
            ret = await self.client.get_historical_consumption(self.start, self.end)
            self.assertEqual(ret.total, 48867.0)

    @patch("ideenergy.Client.is_logged", return_value=True)
    async def test_historical_generation(self, _):
        with patch(
            "ideenergy.Client.request_bytes",
            new_class=AsyncMock,
            side_effect=[read_fixture("historical-generation")],
        ):
            ret = await self.client.get_historical_generation(self.start, self.end)

            self.assertEqual(len(ret.periods), 168)
            self.assertEqual(ret.periods[25].start, datetime(2022, 8, 20, 1, 0))
            self.assertEqual(ret.periods[25].value, 0.0)

    @patch("ideenergy.Client.is_logged", return_value=True)
    async def test_historical_power_demand(self, _):
        with patch(
            "ideenergy.Client.request_bytes",
            new_class=AsyncMock,
            side_effect=[
                read_fixture("historical-power-demand-limits"),
                read_fixture("historical-power-demand"),
            ],
        ):
            ret = await self.client.get_historical_power_demand()

            self.assertEqual(len(ret.demands), 58)
            self.assertEqual(ret.demands[25].dt, datetime(2022, 5, 28, 22, 15))
            self.assertEqual(ret.demands[25].value, 2816.0)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

# Copyright (C) 2021-2022 Luis López <luis@cuarentaydos.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
# USA.


import argparse
import asyncio
import logging
import pprint
import sys
from datetime import datetime, timedelta

from . import Client, RequestFailedError, get_credentials, get_session


def build_arg_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument("-u", "--username", required=False)
    parser.add_argument("-p", "--password", required=False)
    parser.add_argument("--credentials", required=False)
    parser.add_argument("--retries", type=int, default=1)
    parser.add_argument("--contract")

    parser.add_argument("--list-contracts", action="store_true")
    parser.add_argument("--get-measure", action="store_true")
    parser.add_argument("--get-historical-consumption", action="store_true")
    parser.add_argument("--get-historical-generation", action="store_true")
    parser.add_argument("--get-historical-power-demand", action="store_true")

    return parser


async def _main():
    async def get_requested_data():
        if args.list_contracts:
            contracts = await client.get_contracts()
            contracts = {x["codContrato"]: x for x in contracts}
            return contracts

        if args.contract:
            await client.select_contract(args.contract)

        if args.get_measure:
            return await client.get_measure()

        end = datetime.now().replace(hour=0, minute=0, second=0)
        start = end - timedelta(days=7)

        if args.get_historical_consumption:
            return await client.get_historical_consumption(start, end)

        if args.get_historical_generation:
            return await client.get_historical_generation(start, end)

        if args.get_historical_power_demand:
            return await client.get_historical_power_demand()

    logging.basicConfig(
        format="%(asctime)s.%(msecs)03d %(levelname)s %(module)s %(message)s",
        datefmt="%Y-%m-%d %H:%M:%S",
    )
    logger = logging.getLogger("ideenergy")
    logger.setLevel(logging.DEBUG)

    parser = build_arg_parser()
    args = parser.parse_args()
    username, password = get_credentials(args)

    if not username or not password:
        print("Missing username or password", file=sys.stderr)
        sys.exit(1)

    session = await get_session()
    client = Client(
        username=username, password=password, session=session, logger=logger
    )

    try:
        if data := await get_requested_data():
            print(pprint.pformat(data))

    except RequestFailedError as e:
        print(f"Request failed: {e}", file=sys.stderr)
        await session.close()
        sys.exit(1)

    await session.close()


def main():
    return asyncio.run(_main())


if __name__ == "__main__":
    import sys

    sys.exit(main())



# Copyright (C) 2021-2022 Luis López <luis@cuarentaydos.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
# USA.


import itertools
from datetime import datetime, timedelta
from typing import Any

from .types import (
    ConsumptionForPeriod,
    DemandAtInstant,
    HistoricalConsumption,
    HistoricalGeneration,
    HistoricalPowerDemand,
    PeriodValue,
)


def parser_generic_historical_data(data, base_dt: datetime) -> dict[str, Any]:
    def _normalize(idx: int, item: dict | None) -> PeriodValue | None:
        if item is None:
            return None

        start = base_dt + timedelta(hours=idx)
        try:
            return PeriodValue(
                start=start, end=start + timedelta(hours=1), value=float(item["valor"])
            )
        except (KeyError, ValueError, TypeError):
            return None

    g = (_normalize(idx, item) for (idx, item) in enumerate(data["y"]["data"][0]))
    historical_values = [x for x in g if x is not None]

    return {
        # "accumulated": float(data["acumulado"]),
        # "accumulated-co2": float(data["acumuladoCO2"]),
        "historical": historical_values,
    }


def parse_historical_consumption(data) -> HistoricalConsumption:
    def list_to_dict(values, keys):
        return {keys[idx]: values[idx] for idx in range(len(values))}

    start = datetime.strptime(data[0]["fechaDesde"], "%d-%m-%Y").replace(
        hour=0, minute=0, second=0
    )

    period_names = data[0]["periodos"]

    ret = HistoricalConsumption(
        total=data[0]["total"],
        desglosed=list_to_dict(data[0]["totalesPeriodosTarifarios"], period_names),
    )

    for idx, value in enumerate(data[0]["valores"]):
        ret.periods.append(
            ConsumptionForPeriod(
                start=start + timedelta(hours=idx),
                end=start + timedelta(hours=idx + 1),
                value=value,
                desglosed=list_to_dict(
                    data[0]["valoresPeriodosTarifarios"][idx], period_names
                ),
            )
        )

    return ret


def parse_historical_generation(data) -> HistoricalGeneration:
    start = datetime.strptime(data["fechaPeriodo"], "%d-%m-%Y%H:%M:%S").replace(
        hour=0, minute=0, second=0
    )

    parsed = parser_generic_historical_data(data, start)

    return HistoricalGeneration(periods=parsed["historical"])


def parse_historical_power_demand_data(data) -> HistoricalPowerDemand:
    def _normalize_item(item: dict) -> DemandAtInstant:
        return DemandAtInstant(
            dt=datetime.strptime(item["name"], "%d/%m/%Y %H:%M"),
            value=item["y"],
        )

    potMaxMens = data["potMaxMens"]
    potMaxMens = list(itertools.chain.from_iterable([x for x in potMaxMens]))

    demands = [_normalize_item(x) for x in potMaxMens]
    demands = list(sorted(demands, key=lambda x: x.dt))

    return HistoricalPowerDemand(demands=demands)



# Copyright (C) 2021-2022 Luis López <luis@cuarentaydos.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
# USA.


import json
import os
from dataclasses import dataclass, field
from datetime import datetime
from typing import Dict, List

from .client import (
    Client,
    ClientError,
    CommandError,
    InvalidContractError,
    InvalidData,
    RequestFailedError,
    get_session,
)
from .types import (
    ConsumptionForPeriod,
    DemandAtInstant,
    HistoricalConsumption,
    HistoricalGeneration,
    HistoricalPowerDemand,
    Measure,
    PeriodValue,
)


def get_credentials(parsedargs=None, credentials=None, environ_prefix="I_DE_ENERGY"):
    if parsedargs and parsedargs.username:
        return parsedargs.username, parsedargs.password

    credentials = credentials or getattr(parsedargs, "credentials", None)
    if credentials:
        with open(credentials, encoding="utf-8") as fh:
            d = json.loads(fh.read())
        return d["username"], d["password"]

    if environ_prefix:
        environ_prefix = environ_prefix.upper()
        return (
            os.environ.get(f"{environ_prefix}_USERNAME"),
            os.environ.get(f"{environ_prefix}_PASSWORD"),
        )


def sanitize_address(address):
    return " ".join([x.strip().capitalize() for x in address.split(" ") if x])


__all__ = [
    "Client",
    "ClientError",
    "CommandError",
    "get_credentials",
    "get_session",
    "InvalidContractError",
    "InvalidData",
    "RequestFailedError",
    "sanitize_address",
    "ConsumptionForPeriod",
    "DemandAtInstant",
    "HistoricalConsumption",
    "HistoricalGeneration",
    "HistoricalPowerDemand",
    "Measure",
    "PeriodValue",
]

__doc__ = """
    # Returned JSON payloads for login phase:

    scenario: Valid credentials.
    response code: 200
    response json:
    {
        "redirect": "informacion-del-contrato",
        "zona": "B",
        "success": "true",
        "idioma": "ES",
        "uCcr": "",
    }

    scenario: Invalid credentials.
    response code: 200
    response json:
    {
        "success": "false",
        "message": "El usuario o la contraseña que has introducido son incorrectos. Por favor, inténtalo de nuevo.",
    }


    scenario: Several login retries with invalid credentials.
    response code: 200
    response json:
    {
      "success": "false",
      "message": "Número de intentos de acceso excedido. Usuario bloqueado temporalmente. Vuelva a intentarlo dentro de 5 minutos."
    }

    # Retuned JSON payloads for measures (exposed keys: "valLecturaContador", "valMagnitud", "valEstado")

    scenario: valid measure
    response code: 200
    response json:
    {
        "valMagnitud": "10.31",
        "valInterruptor": "1",
        "valEstado": "09",
        "valLecturaContador": "42290",
        "codSolicitudTGT": "009016617100"
    }
    {
        "valMagnitud": "10.49",
        "valInterruptor": "1",
        "valEstado": "09",
        "valLecturaContador": "42290",
        "codSolicitudTGT": "009016672300",
    }

    # Cookies from server:
    [{'version': 0,
      'name': 'JSESSIONID',
      'value': '^0000[0-9a-zA-Z]{18}_[0-9a-zA-Z]{4}:[0-9a-zA-Z]{9}$',
      'port': None,
      'port_specified': False,
      'domain': 'www.i-de.es',
      'domain_specified': False,
      'domain_initial_dot': False,
      'path': '/',
      'path_specified': True,
      'secure': True,
      'expires': None,
      'discard': True,
      'comment': None,
      'comment_url': None,
      'rfc2109': False,
      '_rest': {'HttpOnly': None}},
     {'version': 0,
      'name': 'NSC_wt_mc_mbssvo0Y-[0-9]{5}',
      'value': '^[0-9a-f]{72}$',
      'port': None,
      'port_specified': False,
      'domain': 'www.i-de.es',
      'domain_specified': False,
      'domain_initial_dot': False,
      'path': '/',
      'path_specified': True,
      'secure': True,
      'expires': 1626089796,               # Aprox. 2 minutes
      'discard': False,
      'comment': None,
      'comment_url': None,
      'rfc2109': False,
      '_rest': {'httponly': None}}]
"""
